﻿#================================================================
#
# BOM情報クラス定義
#
# [スクリプト説明]
#   BOMの情報を取得するためのクラスを定義
#
# [注意事項]
#   ・BOM for Windows Ver.8.0がインストールされていること
#   ・BOM for Windows Ver.8.0にのみ対応
#   ・Windows Server 2016以降に対応
#
#================================================================

# BOM監視一覧取得
class BomXMLMonitor {
    [string] $Type
    [string] $ParentId
    [string] $Id
    [string] $Name
    [string] $StartTime
    [string] $Interval
    [string] $IntervalUnit
    [string] $CLSID
    [string] $ObjectName
    [string] $ValueName
    [string] $Options
    [string] $CmpMethodR
    [string] $CmpValueR
    [string] $CmpMethodY
    [string] $CmpValueY
    [string] $Comments
    [string] $IconIndex
    [string] $Enabled
    [string] $DisplayUnit
}

class BomMonitorExecLog {
    [string] $MonitorName
    [string] $Status
    [string] $Value
    [string] $ExecDateTime
    [string] $Code
    [string] $Duration
}

class BomMonitorHistoryLog {
    [string] $Type
    [string] $CreateDateTime
    [string] $Category
    [string] $Summary
}

class BomMonitorInfo {
    [string] $GrpNo
    [string] $MonNo
    [BomXMLMonitor] $Setting
    [BomMonitorExecLog[]] $LogDatas
}

class BomData {
    [BomMonitorInfo[]] $MonitorDatas
    [BomMonitorHistoryLog[]] $HistoryLogs

    BomData()
    {
        # コンストラクタ
        $this.MonitorDatas = @()
        $this.HistoryLogs = @()
    }

    # 値取得
    [string] GetElementValue([object]$element)
    {
        $value = $element
        if ($element.GetType().Name -ne "string") {
            $value = $element."#text"
        }
        return $value
    }

    # BOM監視XML解析
    [BomXMLMonitor] GetBomXMLMonitor([string]$xml_path)
    {
        $obj_xml = $Null

        try
        {
            if((Test-Path $xml_path) -eq $True){
                $xmlData = [XML](Get-Content -Encoding UTF8 -Raw $xml_path)
                $obj_xml = New-Object BomXMLMonitor

                $obj_xml.Type = $this.GetElementValue($xmlData.MonitorItem.Type)
                $obj_xml.ParentId = $this.GetElementValue($xmlData.MonitorItem.ParentId)
                $obj_xml.Id = $this.GetElementValue($xmlData.MonitorItem.Id)
                $obj_xml.Name = $this.GetElementValue($xmlData.MonitorItem.Name)
                $obj_xml.StartTime = $this.GetElementValue($xmlData.MonitorItem.StartTime)
                $obj_xml.Interval = $this.GetElementValue($xmlData.MonitorItem.Interval)
                $obj_xml.IntervalUnit = $this.GetElementValue($xmlData.MonitorItem.IntervalUnit)
                $obj_xml.CLSID = $this.GetElementValue($xmlData.MonitorItem.CLSID)
                $obj_xml.ObjectName = $this.GetElementValue($xmlData.MonitorItem.ObjectName)
                $obj_xml.ValueName = $this.GetElementValue($xmlData.MonitorItem.ValueName)
                $obj_xml.Options = $this.GetElementValue($xmlData.MonitorItem.Options)
                $obj_xml.CmpMethodR = $this.GetElementValue($xmlData.MonitorItem.CmpMethodR)
                $obj_xml.CmpValueR = $this.GetElementValue($xmlData.MonitorItem.CmpValueR)
                $obj_xml.CmpMethodY = $this.GetElementValue($xmlData.MonitorItem.CmpMethodY)
                $obj_xml.CmpValueY = $this.GetElementValue($xmlData.MonitorItem.CmpValueY)
                $obj_xml.Comments = $this.GetElementValue($xmlData.MonitorItem.Comments)
                $obj_xml.IconIndex = $this.GetElementValue($xmlData.MonitorItem.IconIndex)
                $obj_xml.Enabled = $this.GetElementValue($xmlData.MonitorItem.Enabled)
                $obj_xml.DisplayUnit = $this.GetElementValue($xmlData.MonitorItem.DisplayUnit)
            }
        } catch {
            Write-Host "例外: $_"
        }

        return $obj_xml
    }

    # BOM監視ログ解析
    [BomMonitorExecLog[]] GetBomMonitorLog([string]$log_path, [int]$top_count)
    {
        $obj_csv = $Null

        if ($top_count -ge 0) {
            try
            {
                if((Test-Path $log_path) -eq $True){
                    $csv_data = Import-Csv -Path $log_path -Encoding Default -Header "MonitorName", "Status", "Value", "ExecDateTime", "Code", "Duration"
                    if ($top_count -eq 0) {
                        $get_count = $csv_data.Count
                    }
                    else {
                        $get_count = $top_count + 1
                        if ($csv_data.Count -lt $get_count) {
                            $get_count = $csv_data.Count
                        }
                    }

                    $obj_csv = @()
                    for($i = 1; $i -lt $get_count; $i++) {
                        $log_data = New-Object BomMonitorExecLog
                        $log_data.MonitorName = $csv_data[$i].MonitorName
                        $log_data.Status = $csv_data[$i].Status
                        $log_data.Value = $csv_data[$i].Value
                        $log_data.ExecDateTime = $csv_data[$i].ExecDateTime
                        $log_data.Code = $csv_data[$i].Code
                        $log_data.Duration = $csv_data[$i].Duration
                        $obj_csv += $log_data
                    }
                }
            } catch {
                Write-Host "例外: $_"
            }
        }

        return $obj_csv
    }

    # BOM監視ログ出力
    [bool] OutBomMonitorLog([string]$bom_bin_path, [string]$out_path, [string]$instance_id, [string]$grp_id, [string]$mon_id, [int]$duration)
    {
        $result = $FALSE

        $opt_duration = ""
        if ($duration -gt 0) {
            $opt_duration = " -d:"+ $duration
        }

        $exe_path = Join-Path $bom_bin_path "BomCmdLogtxt.exe"
        $exe_param = "-i:" + $instance_id + " -m -gid:" + $grp_id + " -mid:" + $mon_id + $opt_duration +  " -f:""" + $out_path + """"
   
        #Write-Host "CMD: $exe_path $exe_param" 
        try {
            $ret_val = Start-Process -FilePath $exe_path -ArgumentList $exe_param -PassThru -Wait -WindowStyle Hidden
            if ($ret_val.ExitCode -eq 0) {
                $result = $TRUE
            }
            else {
                Write-Host "ExitCode:" $ret_val.ExitCode
            }
        } catch {
            Write-Host "例外: $_"
        }
        return $result
    }

    # BOMヒストリーログ解析
    [BomMonitorHistoryLog[]] GetBomHistoryLog([string]$log_path, [int]$top_count)
    {
        $obj_csv = $Null

        if ($top_count -ge 0) {
            try
            {
                if((Test-Path $log_path) -eq $True){
                    $csv_data = Import-Csv -Path $log_path -Encoding Default -Header "Type", "CreateDateTime", "Category", "Summary"
                    if ($top_count -eq 0) {
                        $get_count = $csv_data.Count
                    }
                    else {
                        $get_count = $top_count + 1
                        if ($csv_data.Count -lt $get_count) {
                            $get_count = $csv_data.Count
                        }
                    }

                    $obj_csv = @()
                    for($i = 1; $i -lt $get_count; $i++) {
                        $log_data = New-Object BomMonitorHistoryLog
                        $log_data.Type = $csv_data[$i].Type
                        $log_data.CreateDateTime = $csv_data[$i].CreateDateTime
                        $log_data.Category = $csv_data[$i].Category
                        $log_data.Summary = $csv_data[$i].Summary
                        $obj_csv += $log_data
                    }
                }
            } catch {
                Write-Host "例外: $_"
            }
        }

        return $obj_csv
    }

    # BOMヒストリーログ出力
    [bool] OutBomHistoryLog([string]$bom_bin_path, [string]$out_path, [string]$kind, [string]$instance_id, [int]$duration)
    {
        $result = $False
        $opt_kind = " -h"
        if ($kind.Length -gt 0) {
            $opt_kind = " -h:"+ $kind
        }
        $exe_path = Join-Path $bom_bin_path "BomCmdLogtxt.exe"
        $exe_param = "-i:" + $instance_id + $opt_kind + " -d:" + [string]$duration + " -f:""" + $out_path + """"
   
        # Write-Host "CMD: $exe_path $exe_param" 
        try {
            $ret_val = Start-Process -FilePath $exe_path -ArgumentList $exe_param -PassThru -Wait -WindowStyle Hidden
            if ($ret_val.ExitCode -eq 0) {
                $result = $True
            }
            else {
                Write-Host "ExitCode:" $ret_val.ExitCode
            }
        } catch {
            Write-Host "例外: $_"
        }
        return $result
    }

    # BOM監視情報取得
    [bool] GetBomXMLData($bom_data_path, $instance_id)
    {
        $result = $False
        $current_path = "Environment\Instance\" + $instance_id + "\Monitor"
        $monitor_path = Join-Path $bom_data_path $current_path

        if (Test-Path $monitor_path -PathType Container) {
            # フォルダ階層を取得
            try {
                $files = Get-ChildItem -Path $monitor_path -File -Recurse -Filter MON*.xml

                # フォルダ情報を表示
                foreach ($file in $files) {
                    $fileName = [System.IO.Path]::GetFileName([string]$file.Name)
                    $folderName = [System.IO.Path]::GetFileName([string]$file.DirectoryName)

                    $ret_val = [regex]::Match($fileName, "^(MON[0-9]{2})\.xml$")
                    if ($ret_val.Success -eq $True) {
                        # 監視XML読み込み
                        $monitor_xml_data = $this.GetBomXMLMonitor($file.FullName)
                        if ($monitor_xml_data -ne $Null -and $monitor_xml_data.Type -eq "Service") {
                            #Write-Host $folderName $fileName
                            $bom_monitor_info = New-Object BomMonitorInfo
                            $bom_monitor_info.GrpNo = $folderName
                            $bom_monitor_info.MonNo = [System.IO.Path]::GetFileNameWithoutExtension($fileName)
                            $bom_monitor_info.Setting = $monitor_xml_data
                            $bom_monitor_info.LogDatas = $Null # ひとまずNULL
                            $this.MonitorDatas += $bom_monitor_info
                        }
                        continue;
                    }
                    else {
                        #Write-Host "対象外:" $fileName
                    }
                }
                $result = $True
            } catch {
                Write-Host "例外: $_"
            }        
        }

        return $result
    }

    # BOMログ取得（監視ログ、ヒストリーログ）
    # ※$MonitorDatasに登録している監視内容をベースにログを追加する。
    [bool] GetBomHistoryData([string]$bom_bin_path, [string]$instance_id, [string]$kind, [int]$monitor_count, [int]$history_duration)
    {
        $result = $False

        try
        {
            # BOM監視ログ追加
            foreach($item in $this.MonitorDatas) {
                # 一時ファイル名
                $out_path = ".\monitor_log_" + $item.GrpNo + $item.MonNo + ".log"
                # BOM監視ログ出力
                $this.OutBomMonitorLog($bom_bin_path, $out_path, $instance_id, $item.GrpNo, $item.MonNo, 0)
                # BOM監視ログ取得
                $item.LogDatas = $this.GetBomMonitorLog($out_path, $monitor_count)

                # 一時ファイル削除
                Remove-Item -Path $out_path -Force
            }
            
            # BOMヒストリーログ追加
            # 一時ファイル名
            $out_path = ".\history_log.log"
            # BOMヒストリーログ出力
            $this.OutBomHistoryLog($bom_bin_path, $out_path, $kind, $instance_id, $history_duration)
            # BOMヒストリーログ取得
            $this.HistoryLogs = $this.GetBomHistoryLog($out_path, 0)

            # 一時ファイル削除
            Remove-Item -Path $out_path -Force

            $result = $True
        } catch {
            Write-Host "例外: $_"
        }
        
        return $result
    }
}
